package net.java.dev.csseditor;

import java.awt.BorderLayout;
import java.awt.Component;

import java.beans.PropertyVetoException;

import java.util.Map;

import javax.swing.JDesktopPane;
import javax.swing.JInternalFrame;
import javax.swing.JMenuItem;
import javax.swing.JOptionPane;
import javax.swing.JPanel;

import net.java.dev.csseditor.actions.ChangeWindow;
import net.java.dev.csseditor.EditorFrame;
import net.java.dev.csseditor.EditorMenuBar;
import net.java.dev.csseditor.EditorToolBar;
import net.java.dev.csseditor.jedit.JEditTextWriter;
import net.java.dev.csseditor.jedit.JEditValidator;
import net.java.dev.csseditor.PreviewPanel;
import net.java.dev.csseditor.StyleParser;
import net.java.dev.csseditor.TextWriter;

import org.gjt.sp.jedit.View;

/**
 * The main editor class which allows users to create css style elements. Brings
 * up the main panel, adds all tab panels and a menu, and allows creation of new
 * css elements and entering them in the current buffer
 *
 * @author   <a href="mailto:uzi@landsmanns.com">Uzi Landsmann</a>
 */
public class CssEditor extends JPanel {
	private static View view;

	private PreviewPanel preview = new PreviewPanel();
	private TextWriter writer;
	private StyleParser parser;
	private JDesktopPane desktop = new JDesktopPane();
	private EditorMenuBar menuBar = new EditorMenuBar(this);
	private EditorToolBar toolBar = new EditorToolBar(this);
	private static CssEditor cssEditor;


	/*
	 *  CssEditor view structure:
	 *  ___________________________
	 *  |__________________________ |
	 *  ||JPanel top               ||
	 *  || _______________________ ||
	 *  |||EditorMenuBar          |||
	 *  |||_______________________|||
	 *  || _______________________ ||
	 *  |||JToolBar               |||
	 *  |||_______________________|||
	 *  ||_________________________||
	 *  | _________________________ |
	 *  ||JPanel desktop           ||
	 *  ||  _________________      ||
	 *  || |EditorFrame frame|     ||
	 *  || |                 |     ||
	 *  || |_________________|_    ||
	 *  ||   |EditorFrame frame|   ||
	 *  ||   |                 |   ||
	 *  ||   |_________________|   ||
	 *  ||_________________________||
	 *  |__________________________ |
	 *  ||JPanel preview           ||
	 *  ||_________________________||
	 *  |___________________________|
	 */
	/**
	 * Constructor for the CssEditor object
	 *
	 * @param view  The jEdit's View
	 */
	public CssEditor(View view) {
		this.view = view;
		cssEditor = this;
		desktop.setDragMode(JDesktopPane.OUTLINE_DRAG_MODE);
		setLayout(new BorderLayout());
		writer = new JEditTextWriter(view);
		parser = new StyleParser(this);

		// button and name on top panel
		JPanel top = new JPanel(new BorderLayout());

		top.add(menuBar, BorderLayout.NORTH);
		top.add(toolBar, BorderLayout.CENTER);

		add(top, BorderLayout.NORTH);
		add(desktop, BorderLayout.CENTER);
		add(preview, BorderLayout.SOUTH);
	}


	/**
	 * Gets the editor object
	 *
	 * @return   the editor object
	 */
	public static CssEditor getEditor() {
		return cssEditor;
	}


	/**
	 * Gets the css <code>StyleParser</code> object
	 *
	 * @return   The css <code>StyleParser</code> object
	 */
	public StyleParser getParser() {
		return parser;
	}


	/**
	 * Gets the selected <code>EditorFrame</code>
	 *
	 * @return   the selected <code>EditorFrame</code>
	 */
	public EditorFrame getEditorFrame() {
		return (EditorFrame) desktop.getSelectedFrame();
	}


	/**
	 * Updates the <code>PreviewPanel</code> to show a css element with current
	 * selected attributes.
	 *
	 * @see   PreviewPanel
	 */
	public void updatePreview() {
		EditorFrame frame = getEditorFrame();

		if (frame != null) {
			resetPreview();
			preview.update(frame.getElementName(), frame.getProps());
		}
	}


	/**
	 * Resets the <code>PreviewPanel</code> which is used to show a css element
	 * with current selected attributes.
	 *
	 * @see   PreviewPanel
	 */
	public void resetPreview() {
		remove(preview);
		preview = new PreviewPanel();
		add(preview, BorderLayout.SOUTH);
	}


	/**
	 * Gets all field values and stores them in a <code>Map</code>, then calls
	 * <code>TextWriter</code> which writes the css element in the current jEdit
	 * buffer
	 *
	 * @param props        the <code>Map</code> which stroes all field values
	 * @param elementName  the current element name
	 */
	public void printAllFields(String elementName, Map props) {
		writer.writeNewStyleItem(elementName, props);
	}


	/**
	 * Gets the current jEdit view
	 *
	 * @return   the current jEdit view
	 */
	public static View getView() {
		return view;
	}


	/**
	 * Gets the desktop attribute of the CssEditor object
	 *
	 * @return   The desktop value
	 */
	public JDesktopPane getDesktop() {
		return desktop;
	}


	/**
	 * Gets the preview panel
	 *
	 * @return   the preview panel
	 * @see      PreviewPanel
	 */
	public PreviewPanel getPreview() {
		return preview;
	}


	/**
	 * Creates a new <code>EditorFrame</code> object for a new css element.
	 *
	 * @param elementName  the new element's name
	 * @param props        the list of css attributes for the element
	 */
	public void createNewFrame(String elementName, Map props) {
		EditorFrame frame = new EditorFrame(cssEditor, elementName, desktop.getSize(), props);
		frame.setName(elementName);
		frame.setVisible(true);
		desktop.add(frame);
		try {
			frame.setSelected(true);
		} catch (PropertyVetoException pve) {
			pve.printStackTrace();
		}

		JMenuItem window = new JMenuItem(elementName);
		window.setAction(new ChangeWindow(this, elementName));
		window.setName(elementName);
		window.getAccessibleContext().setAccessibleDescription(elementName);
		menuBar.getWindows().add(window);
		toolBar.getWindowsCombo().addItem(elementName);
		toolBar.getWindowsCombo().setSelectedItem(elementName);
	}


	/** Creates a frame for an element */
	public void createNewFrame() {
		Object obj = JOptionPane.showInputDialog(this, "Element name:");
		if (obj instanceof String) {
			String elementName = (String) obj;
			createNewFrame(elementName, null);
		}
	}


	/**
	 * Removes an element
	 *
	 * @param frame  the frame to remove
	 */
	public void removeFrame(EditorFrame frame) {
		String frameName = frame.getName();
		Component[] items = menuBar.getWindows().getMenuComponents();
		for (int i = 0; i < items.length; i++) {
			String name = items[i].getName();
			if (name != null && name.equals(frameName)) {
				menuBar.getWindows().remove(i);
				toolBar.getWindowsCombo().removeItem(frameName);
			}
		}
		frame.dispose();
	}


	/**
	 * Select an EditorFrame with the given name.
	 *
	 * @param name  The frame's name
	 */
	public void changeFrame(String name) {
		JInternalFrame[] frames = desktop.getAllFrames();
		int i;
		for (i = 0; i < frames.length; i++) {
			if (frames[i].getTitle().equals(name)) {
				break;
			}
		}

		try {
			frames[i].setSelected(true);
			toolBar.getWindowsCombo().setSelectedItem(name);
		} catch (PropertyVetoException pve) {
			pve.printStackTrace();
		}
	}


	/**
	 * Validate the file in the current <code>Buffer</code> using the W3C Css
	 * Validator. Any errors or warnings that the generator might found will be
	 * reported using the <code>ErrorList plugin</code>.
	 *
	 * @see   <a href="http://jigsaw.w3.org/css-validator/">W3C Css Validator</a>
	 * @see   <a href="http://plugins.jedit.org/plugins/?ErrorList">Error List
	 *      Plugin</a>
	 */
	public void validateFile() {
		new JEditValidator().val(view.getBuffer().getPath());
	}
}

