package net.java.dev.csseditor;

import java.awt.BorderLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import java.util.Iterator;
import java.util.Map;
import java.util.Set;
import java.util.TreeMap;

import javax.swing.event.InternalFrameAdapter;
import javax.swing.event.InternalFrameEvent;
import javax.swing.JButton;
import javax.swing.JInternalFrame;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTabbedPane;

import net.java.dev.csseditor.CssEditor;
import net.java.dev.csseditor.tabs.AbstractTabPanel;
import net.java.dev.csseditor.tabs.Background;
import net.java.dev.csseditor.tabs.Border;
import net.java.dev.csseditor.tabs.Classification;
import net.java.dev.csseditor.tabs.Content;
import net.java.dev.csseditor.tabs.Dimension;
import net.java.dev.csseditor.tabs.Font;
import net.java.dev.csseditor.tabs.List;
import net.java.dev.csseditor.tabs.Margin;
import net.java.dev.csseditor.tabs.Outline;
import net.java.dev.csseditor.tabs.Padding;
import net.java.dev.csseditor.tabs.Position;
import net.java.dev.csseditor.tabs.Table;
import net.java.dev.csseditor.tabs.Text;

/**
 * A frame inside the editor's desktop which represents one element and contains
 * tabs with the element's properties
 *
 * @author   <a href="mailto:uzi@landsmanns.com">uzi@landsmanns.com</a>
 */
public class EditorFrame extends JInternalFrame {
	private Map props = new TreeMap();

	private JButton addButton = new JButton("Add");
	private JButton clearButton = new JButton("Clear");

	private AbstractTabPanel background = new Background(this);
	private AbstractTabPanel border = new Border(this);
	private AbstractTabPanel classification = new Classification(this);
	private AbstractTabPanel content = new Content(this);
	private AbstractTabPanel dimension = new Dimension(this);
	private AbstractTabPanel font = new Font(this);
	private AbstractTabPanel list = new List(this);
	private AbstractTabPanel margin = new Margin(this);
	private AbstractTabPanel outline = new Outline(this);
	private AbstractTabPanel padding = new Padding(this);
	private AbstractTabPanel position = new Position(this);
	private AbstractTabPanel table = new Table(this);
	private AbstractTabPanel text = new Text(this);

	private Map tabPanels = new TreeMap();
	private JTabbedPane tabs;
	private CssEditor editor;
	private String elementName;

	private EditorFrame frame = this;


	/**
	 * Constructor for the EditorFrame object
	 *
	 * @param editor       Description of the Parameter
	 * @param elementName  Description of the Parameter
	 * @param size         Description of the Parameter
	 * @param props        Description of the Parameter
	 */
	public EditorFrame(CssEditor editor, String elementName, java.awt.Dimension size, Map props) {
		this(editor, elementName, size);
		if (props != null) {
			setProps(props);
		}
	}


	/**
	 * Constructor for the CssEditor object
	 *
	 * @param editor       the editor
	 * @param elementName  the element's name
	 * @param size         the frame's size
	 */
	public EditorFrame(CssEditor editor, String elementName, java.awt.Dimension size) {
		super(elementName, true, true, true, true);
		this.editor = editor;
		this.elementName = elementName;
		getContentPane().setLayout(new BorderLayout());

		// button and name on top panel
		JPanel top = new JPanel();
		addButton.setActionCommand("add");
		clearButton.setActionCommand("clear");
		ButtonListener bl = new ButtonListener();
		addButton.addActionListener(bl);
		clearButton.addActionListener(bl);
		top.add(addButton);
		top.add(clearButton);

		//add tabs panel to a vector
		tabPanels.put("Background", background);
		tabPanels.put("Border", border);
		tabPanels.put("Classification", classification);
		tabPanels.put("Content", content);
		tabPanels.put("Dimension", dimension);
		tabPanels.put("Font", font);
		tabPanels.put("List", list);
		tabPanels.put("Margin", margin);
		tabPanels.put("Outline", outline);
		tabPanels.put("Padding", padding);
		tabPanels.put("Position", position);
		tabPanels.put("Table", table);
		tabPanels.put("Text", text);

		// tabs on center panel
		tabs = new JTabbedPane(JTabbedPane.TOP, JTabbedPane.WRAP_TAB_LAYOUT);
		Set names = tabPanels.keySet();
		Iterator iter = names.iterator();
		while (iter.hasNext()) {
			String name = (String) iter.next();
			AbstractTabPanel tabPanel = (AbstractTabPanel) tabPanels.get(name);
			tabs.add(name, new JScrollPane(tabPanel));
		}
		getContentPane().add(top, BorderLayout.SOUTH);
		getContentPane().add(tabs, BorderLayout.CENTER);

		addInternalFrameListener(
			new InternalFrameAdapter() {
				public void internalFrameActivated(InternalFrameEvent e) {
					CssEditor.getEditor().updatePreview();
				}


				public void internalFrameClosing(InternalFrameEvent e) {
					CssEditor.getEditor().removeFrame(frame);
				}
			});

		setSize(size);
	}


	/**
	 * Calls every tab panel and calls its clearFields method to clear its fields
	 */
	private void clearAllFields() {
		Set names = tabPanels.keySet();
		Iterator iter = names.iterator();
		while (iter.hasNext()) {
			String name = (String) iter.next();
			AbstractTabPanel tabPanel = (AbstractTabPanel) tabPanels.get(name);
			tabPanel.clearFields();
		}
		props.clear();
	}


	/**
	 * Calls every tab panel and calls its getFields method to store all field
	 * values in the map
	 */
	private void getAllFields() {
		Set names = tabPanels.keySet();
		Iterator iter = names.iterator();
		while (iter.hasNext()) {
			String name = (String) iter.next();
			AbstractTabPanel tabPanel = (AbstractTabPanel) tabPanels.get(name);
			tabPanel.getFields(props);
		}
	}


	/**
	 * Gets a <code>Map</code> with the elements properties and values
	 *
	 * @return   a <code>Map</code> with the elements properties and values
	 */
	public Map getProps() {
		getAllFields();
		return props;
	}


	/**
	 * Sets the props attribute of the EditorFrame object
	 *
	 * @param props  The new props value
	 */
	public void setProps(Map props) {
		Set names = tabPanels.keySet();
		Iterator iter = names.iterator();

		while (iter.hasNext()) {
			String name = (String) iter.next();
			AbstractTabPanel tabPanel = (AbstractTabPanel) tabPanels.get(name);
			tabPanel.setProps(props);
		}
	}


	/**
	 * Gets the element name
	 *
	 * @return   the element name
	 */
	public String getElementName() {
		return elementName;
	}


	/**
	 * Sets the element name
	 *
	 * @param elementName  the new element name
	 */
	public void setElementName(String elementName) {
		this.elementName = elementName;
		setTitle(elementName);
		editor.updatePreview();
	}


	/**
	 * A listener to button events
	 *
	 * @author   <a href="mailto:uzi@landsmanns.com">uzi@landsmanns.com</a>
	 */
	class ButtonListener implements ActionListener {

		/**
		 * Listens to action buttons
		 *
		 * @param ae  the action event
		 */
		public void actionPerformed(ActionEvent ae) {
			String which = ae.getActionCommand();
			if (which.equals("add")) {
				editor.printAllFields(elementName, props);
			} else if (which.equals("clear")) {
				clearAllFields();
			}
		}
	}
}

