package net.java.dev.csseditor;

import java.awt.BorderLayout;

import java.util.Iterator;
import java.util.Map;
import java.util.Set;

import javax.swing.BorderFactory;
import javax.swing.JEditorPane;
import javax.swing.JPanel;

/**
 * A panel that shows a preview of the edited css element
 *
 * @author   <a href="mailto:uzi@landsmanns.com">uzi@landsmanns.com</a>
 */
public class PreviewPanel extends JPanel {

	private JEditorPane editor = new JEditorPane();
	/** A list of table-related elements */
	public static String[] tableElements = {"caption", "col", "colgroup", "table", "tbody", "td", "tfoot", "th", "thead", "tr"};
	/** A list of list-related elements */
	public static String[] listElements = {"dd", "dir", "dl", "dt", "li", "menu", "ol", "ul"};
	/** A list of form-related elements */
	public static String[] formElements = {"button", "form", "fieldset", "input", "isindex", "label", "legend", "optgroup", "option", "select", "textarea"};
	/** A list of html formating elements */
	public static String[] formatingElements = {"a", "abbr", "acronym", "address", "b", "big", "blockquote", "cite", "code", "del", "dfn", "div", "em", "font", "h1", "h2", "h3", "h4", "h5", "i", "ins", "kbd", "p", "pre", "q", "s", "samp", "small", "span", "strike", "strong", "sub", "sup", "tt", "u"};
	/** A list of diverse elements */
	public static String[] diverseElements = {"br", "hr", "img"};


	/** Constructor for the PreviewPanel object */
	public PreviewPanel() {
		setLayout(new BorderLayout());
		editor.setContentType("text/html");
		editor.setEditable(false);
		editor.setBorder(BorderFactory.createTitledBorder("Preview"));
		reset();

		add(editor, BorderLayout.CENTER);
	}


	/** Resets the panel to show nothing */
	public void reset() {
		editor.setText("<html><body>&nbsp;</body></html>");
	}


	/**
	 * Updates the element preview
	 *
	 * @param elementName  the elements name
	 * @param props        the list of element's css property names and values
	 */
	public void update(String elementName, Map props) {
		String html = "<html><head><style>\n";
		html += elementName + " {\n";
		Set keys = props.keySet();
		Iterator iter = keys.iterator();
		while (iter.hasNext()) {
			String key = "" + iter.next();
			html += "\t" + key + ": " + props.get(key) + ";\n";
		}
		html += "}\n</style></head>\n";
		html += parseName(elementName);
		html += "</body></html>";

		editor.setText(html);
	}


	/**
	 * Parses the element's name and adds corresponding html tags
	 *
	 * @param name  the element's name
	 * @return      html tags which correspond to the element's name
	 */
	private String parseName(String name) {
		int dot = name.indexOf('.');
		int hash = name.indexOf('#');
		int i = Math.max(dot, hash);

		String html = "";
		String className = "";
		String forName = name;

		if (i > -1 && name.length() > i) {
			if (dot > hash) {
				className = " class='" + name.substring(i + 1) + "'";
			} else {
				className = " id='" + name.substring(i + 1) + "'";
			}
			forName = name.substring(0, i);
		}

		int col = forName.indexOf(':');
		if (col > -1) {
			forName = forName.substring(0, col);
		}

		int formatingIndex = findIndex(formatingElements, forName);
		int listIndex = findIndex(listElements, forName);
		int tableIndex = findIndex(tableElements, forName);
		int formIndex = findIndex(formElements, forName);
		int diverseIndex = findIndex(diverseElements, forName);

		if (name.startsWith(".")) {
			html += "<body>\n<div" + className + ">" + name + "</div>\n";
			return html;
		}

		if (forName.startsWith("body")) {
			html += "<body" + className + ">\n";
			html += name + "\n";
			return html;
		}

		html += "<body>\n";

		if (formatingIndex > -1) {
			html += "<" + formatingElements[formatingIndex] + className + ">";
			html += name;
			html += "</" + formatingElements[formatingIndex] + ">\n";
			return html;
		}

		if (listIndex > -1) {
			if (forName.equalsIgnoreCase("ul") || forName.equalsIgnoreCase("ol") ||
					forName.equalsIgnoreCase("menu") || forName.equalsIgnoreCase("dir")) {
				html += "<" + forName + className + "><li>" + name + "</li></" + forName + ">\n";
			} else if (forName.equalsIgnoreCase("dl")) {
				html += "<" + forName + className + "><dt>element</dt><dd>" + name + "</dd></" + forName + ">\n";
			} else if (forName.equalsIgnoreCase("li")) {
				html += "<ul><li" + className + ">" + name + "</li></ul>\n";
			} else if (forName.equalsIgnoreCase("dt")) {
				html += "<dl><dt" + className + ">" + name + "</dt><dd>definition</dd></dl>\n";
			} else if (forName.equalsIgnoreCase("dd")) {
				html += "<dl><dt>element</dt><dd" + className + ">" + name + "</dd></dl>\n";
			}
			return html;
		}

		if (tableIndex > -1) {
			if (forName.equalsIgnoreCase("table")) {
				html += "<table" + className + "><tr><td>" + name + "</td></tr></table>\n";
			} else if (forName.equalsIgnoreCase("thead") || forName.equalsIgnoreCase("tbody") || forName.equalsIgnoreCase("tfoot")) {
				html += "<table><" + forName + className + "><tr><td>" + name + "</td></tr></" + forName + "></table>\n";
			} else if (forName.equalsIgnoreCase("th") || forName.equalsIgnoreCase("td")) {
				html += "<table><tr><" + forName + className + ">" + name + "</" + forName + "></tr></table>\n";
			} else if (forName.equalsIgnoreCase("tr")) {
				html += "<table><tr" + className + "><td>" + name + "</td></tr></table>\n";
			} else if (forName.equalsIgnoreCase("caption")) {
				html += "<table><caption" + className + ">" + name + "</caption>" +
						"<tr><td>table</td></tr></table>\n";
			} else if (forName.equalsIgnoreCase("colgroup")) {
				html += "<table><colgroup" + className + ">" + name + "<col></col></colgroup>" +
						"<tr><td>table</td></tr></table>\n";
			} else if (forName.equalsIgnoreCase("col")) {
				html += "<table><colgroup><col" + className + ">" + name + "</col></colgroup>" +
						"<tr><td>table</td></tr></table>\n";
			}
			return html;
		}

		if (formIndex > -1) {
			if (forName.equalsIgnoreCase("form")) {
				html += "<form" + className + ">" + name + ":&nbsp;<input type='text'></form>";
			} else if (forName.equalsIgnoreCase("input") || forName.equalsIgnoreCase("isindex")) {
				html += "<form>" + name + ":&nbsp;<" + forName + " type='text'" + className + "></form>";
			} else if (forName.equalsIgnoreCase("textarea")) {
				html += "<form><textarea" + className + ">" + name + "</textarea></form>";
			} else if (forName.equalsIgnoreCase("label")) {
				html += "<form><label for='abc'" + className + ">" + name + ":</label>" +
						"<input type='text' id='abc'></form>";
			} else if (forName.equalsIgnoreCase("fieldset")) {
				html += "<form><fieldset" + className + ">" + name + ":&nbsp;<input type='text'></fieldset></form>";
			} else if (forName.equalsIgnoreCase("legend")) {
				html += "<form><fieldset><legend" + className + ">" + name +
						"</legend>input:&nbsp;<input type='text'></fieldset></form>";
			} else if (forName.equalsIgnoreCase("select")) {
				html += "<form><select" + className + "><option>" + name + "</option></select></form>";
			} else if (forName.equalsIgnoreCase("optgroup")) {
				html += "<form><select><optgroup" + className + " label='" + name + "'><option>option</option></optgroup></select></form>";
			} else if (forName.equalsIgnoreCase("option")) {
				html += "<form><select><option" + className + ">" + name + "</option></select></form>";
			} else if (forName.equalsIgnoreCase("button")) {
				html += "<form><button" + className + ">" + name + "</button></form>";
			}
			return html;
		}

		if (diverseIndex > -1) {
			html += "<" + forName + className + ">";
			return html;
		}

		return html;
	}


	/**
	 * Finds a given string index in an array
	 *
	 * @param strings   the list of strings to look through
	 * @param toSearch  the string to find
	 * @return          the index of the string in the array, or -1 if it was not
	 *      found
	 */
	private int findIndex(String[] strings, String toSearch) {
		int i;
		for (i = 0; i < strings.length; i++) {
			if (toSearch.equalsIgnoreCase(strings[i])) {
				return i;
			}
		}
		return -1;
	}
}


