package net.java.dev.csseditor.listeners;

import java.awt.event.ActionListener;

import javax.swing.JComboBox;

import net.java.dev.csseditor.CssEditor;
import net.java.dev.csseditor.tabs.AbstractTabPanel;

/**
 * An abstract <code>Listener</code> to CSS attribute <code>JComboBox</code>
 * events. The actionPerformed method is generated only when the user chooses a
 * special value in the <code>JComboBox</code>. When this happens, this class
 * will pop-up an apropriate <code>Chooser</code> to choose values from.
 *
 * @author   <a href="mailto:uzi@landsmanns.com">Uzi Landsmann</a>
 * @see      net.java.dev.csseditor.choosers.AbstractChooser
 * @see      net.java.dev.csseditor.choosers.DoublePercentChooser
 * @see      net.java.dev.csseditor.choosers.DoubleSizeUnitChooser
 * @see      net.java.dev.csseditor.choosers.FileChooser
 * @see      net.java.dev.csseditor.choosers.PercentChooser
 * @see      net.java.dev.csseditor.choosers.ShapeChooser
 * @see      net.java.dev.csseditor.choosers.SizeChooser
 * @see      net.java.dev.csseditor.choosers.SizeUnitChooser
 * @see      net.java.dev.csseditor.choosers.StringChooser
 */
public abstract class AbstractListener implements ActionListener {

	/** The <code>JPanel</code> which uses this listener */
	protected AbstractTabPanel owner;
	/** The <code>JComboBox</code> to listen to */
	protected JComboBox combo;
	/** The <code>Chooser</code>'s title */
	protected String title;
	/** The value that this listener is waiting for */
	protected String word;


	/**
	 * Constructor for the <code>ColorListener</code> object
	 *
	 * @param owner  The <code>JPanel</code> which uses this Listener
	 * @param title  The <code>Chooser</code>'s title
	 * @param word   The value that this listener is waiting for
	 */
	public AbstractListener(AbstractTabPanel owner, String title, String word) {
		this.owner = owner;
		this.title = title;
		this.word = word;
	}


	/**
	 * Add a given value in the <code>JComboBox</code> and choose it
	 *
	 * @param obj  The new value to add
	 */
	public void setValues(Object obj) {
		String values = "" + obj;
		combo.addItem(values);
		combo.setSelectedItem(values);
	}


	/**
	 * Add given size and unit values as a single value in the <code>JComboBox</code>
	 * and choose it.
	 *
	 * @param size  The new size value
	 * @param unit  The new unit
	 */
	public void setValues(Object size, Object unit) {
		String values = "" + size + unit;
		combo.addItem(values);
		combo.setSelectedItem(values);
	}


	/**
	 * Add given coordinates and unit values as a single value in the <code>JComboBox</code>
	 * and choose it.
	 *
	 * @param x     The new x coordinate value
	 * @param y     The new y coordinate value
	 * @param unit  The new unit
	 */
	public void setValues(Object x, Object y, String unit) {
		String values = "" + x + unit + " " + y + unit;
		combo.addItem(values);
		combo.setSelectedItem(values);
	}


	/**
	 * Add given double sizes and units values as a single value in the <code>JComboBox</code>
	 * and choose it.
	 *
	 * @param x      The new x value
	 * @param xUnit  The new x unit
	 * @param y      The new y value
	 * @param yUnit  The new y unit
	 */
	public void setValues(Object x, Object xUnit, Object y, Object yUnit) {
		String values = "" + x + xUnit + " " + y + yUnit;
		combo.addItem(values);
		combo.setSelectedItem(values);
	}


	/**
	 * Add given quadruple sizes and units values as a single value in the <code>JComboBox</code>
	 * and choose it.
	 *
	 * @param a      The new first value
	 * @param aUnit  The new first unit
	 * @param b      The new second value
	 * @param bUnit  The new second unit
	 * @param c      The new third value
	 * @param cUnit  The new third unit
	 * @param d      The new forth value
	 * @param dUnit  The new forth unit
	 */
	public void setValues(Object a, Object aUnit, Object b, Object bUnit,
			Object c, Object cUnit, Object d, Object dUnit) {
		String values = "rect(" + a + aUnit + " " + b + bUnit + " " + c + cUnit + " " + d + dUnit + ")";
		combo.addItem(values);
		combo.setSelectedItem(values);
	}


	/**
	 * Add given list of values in the <code>JComboBox</code>
	 *
	 * @param strings  The new list of values
	 */
	public void setValues(String[] strings) { }


	/** Choose an empty value in the <code>JComboBox</code> */
	public void resetValues() {
		combo.setSelectedItem("");
	}


	/**
	 * Update the <code>PreviewPanel</code>
	 *
	 * @see   net.java.dev.csseditor.PreviewPanel
	 */
	protected void updatePreview() {
		CssEditor editor = CssEditor.getEditor();
		editor.updatePreview();
	}
}

